using UnityEngine;
using System.Collections.Generic;
using HarmonyLib;
using Steamworks;
using UnityEngine.AzureSky;

public class Shark_Attack_Delayed : Mod
{

    private Harmony harmonyInstance;
    public static List<Shark_Attack_Delayed_Stats> allSharkStats;

    public void Start()
    {
        harmonyInstance = new Harmony("com.nephlymio.sharkattackdelayed");
        harmonyInstance.PatchAll();
        allSharkStats = new List<Shark_Attack_Delayed_Stats>();
        Debug.Log("Mod Shark_Attack_Delayed has been loaded!");
    }

    public void OnModUnload()
    {
        allSharkStats.Clear();
        harmonyInstance.UnpatchAll("com.nephlymio.sharkattackdelayed");
        Debug.Log("Mod Shark_Attack_Delayed has been unloaded!");
    }

    public override void WorldEvent_WorldLoaded()
    {
        allSharkStats = new List<Shark_Attack_Delayed_Stats>();
        Network_Player curPlayer = RAPI.GetLocalPlayer();
        allSharkStats.Add(new Shark_Attack_Delayed_Stats(curPlayer));
    }

    public override void WorldEvent_WorldUnloaded()
    {
        allSharkStats.Clear();
    }


    public override void WorldEvent_OnPlayerConnected(CSteamID steamid, RGD_Settings_Character characterSettings)
    {
        Network_Player otherplayer = RAPI.GetLocalPlayer().Network.remoteUsers[steamid];
        allSharkStats.Add(new Shark_Attack_Delayed_Stats(otherplayer));
    }

    public override void WorldEvent_OnPlayerDisconnected(CSteamID steamid, DisconnectReason disconnectReason)
    {
        Network_Player theDisconnectingPlayer = RAPI.GetLocalPlayer().Network.remoteUsers[steamid];
        foreach (Shark_Attack_Delayed_Stats stat in allSharkStats)
        {
            if (stat.player == theDisconnectingPlayer)
            {
                allSharkStats.Remove(stat);
                break;
            }
        }
    }

    public void Update()
    {
        if (Raft_Network.IsHost)
		{
			if(LoadSceneManager.IsGameSceneLoaded)
			{
        
				AzureSkyController sc = ComponentManager<AzureSkyController>.Value;
				int activehour = GetHourString(sc.timeOfDay.hour);
				foreach (Shark_Attack_Delayed_Stats stat in allSharkStats)
				{
					stat.Update(activehour);
				}
			}
		}
    }
 
    static int GetHourString(float TimeOfDay)
    {
        int hour = (int)TimeOfDay;
        return hour;
    }
}

public class Shark_Attack_Delayed_Stats
{
    public float timeInWater;
    public float attackWait;
    public float timeOnLand;
    public bool resetshark;
    public bool sharkCanAttack;
    public Network_Player player;
    public Shark_Attack_Delayed_Stats(Network_Player parent)
    {
        player = parent;
        timeInWater = 0;
        timeOnLand = 0;
        attackWait = 0;
        resetshark = false;
        sharkCanAttack = false;
    }

    public void Update(int activehour)
    {
        
        float timePassed = Time.deltaTime;
        if (player.PersonController.SubmersionState == UltimateWater.SubmersionState.None) // player is on land
        {
            if (timeInWater != 0)
            {
                timeOnLand += timePassed;
                if (timeOnLand > 4) // time on land is more than 4 seconds
                {
                    timeInWater -= timePassed;
                    if (timeInWater <= 0)
                    {
                        timeInWater = 0;
                        resetshark = false;
                    }
                }    
            }
        }
        else // player is not on land i.e. in water
        {
            if (timeOnLand != 0)
                timeOnLand = 0;
            if (timeInWater == 0)
                attackWait = new System.Random().Next(10, 20); // pick a random wait time from 10 to 20 seconds
            
            timeInWater += timePassed;
            if (activehour >= 20 || activehour <= 4) 
            { 
                timeInWater += 2*timePassed; 
            }

            if (timeInWater > attackWait)
            {
                timeInWater = 25;
                resetshark = true;
            }          
            sharkCanAttack = resetshark;                
        }
    }
}

[HarmonyPatch(typeof(Helper), "IsValidWaterTarget")]
public class Patch_ValidWaterTargets
{
    static void Postfix(ref bool __result, ref Network_Player player)
    {
        if (__result)
        {
            foreach (Shark_Attack_Delayed_Stats stat in Shark_Attack_Delayed.allSharkStats)
            {
                if (stat.player == player)
                {
                    __result = stat.sharkCanAttack;
                    break;
                }
            }
        }
    }
}